@@ -326,12 +326,20 @@ css_changed(GtkWidget *self, GtkCssStyleChange *change)
 		->css_changed(self, change);
 
 	CelluloidControlBox *box = CELLULOID_CONTROL_BOX(self);
-	GtkStyleContext *ctx = gtk_widget_get_style_context(self);
-	GtkBorder padding = {0};
+	gint offset = -6;
+	graphene_rect_t bounds;
 
-	gtk_style_context_get_padding(ctx, &padding);
-	g_object_set(box->seek_bar, "popover-y-offset", -padding.top, NULL);
-	g_object_set(box->secondary_seek_bar, "popover-y-offset", -padding.top, NULL);
+	if(gtk_widget_compute_bounds(self, box->seek_bar, &bounds))
+	{
+		offset = (gint)bounds.origin.y;
+	}
+	else
+	{
+		g_warning("Failed to calculate offset for timestamp popover.");
+	}
+
+	g_object_set(box->seek_bar, "popover-y-offset", offset, NULL);
+	g_object_set(box->secondary_seek_bar, "popover-y-offset", offset, NULL);
 }
 
 static gboolean
@@ -428,13 +436,13 @@ set_skip_enabled(CelluloidControlBox *box, gboolean enabled)
 {
 	if(enabled)
 	{
-		gtk_widget_show(box->previous_button);
-		gtk_widget_show(box->next_button);
+		gtk_widget_set_visible(box->previous_button, TRUE);
+		gtk_widget_set_visible(box->next_button, TRUE);
 	}
 	else
 	{
-		gtk_widget_hide(box->previous_button);
-		gtk_widget_hide(box->next_button);
+		gtk_widget_set_visible(box->previous_button, FALSE);
+		gtk_widget_set_visible(box->next_button, FALSE);
 	}
 }
 
@@ -458,15 +466,15 @@ set_compact(CelluloidControlBox *box, gboolean compact)
 	{
 		gtk_widget_set_halign(box->inner_box, GTK_ALIGN_CENTER);
 		gtk_widget_set_margin_start(box->loop_button, 12);
-		gtk_widget_hide(box->seek_bar);
-		gtk_widget_show(box->secondary_seek_bar);
+		gtk_widget_set_visible(box->seek_bar, FALSE);
+		gtk_widget_set_visible(box->secondary_seek_bar, TRUE);
 	}
 	else
 	{
 		gtk_widget_set_halign(box->inner_box, GTK_ALIGN_FILL);
 		gtk_widget_set_margin_start(box->loop_button, 0);
-		gtk_widget_show(box->seek_bar);
-		gtk_widget_hide(box->secondary_seek_bar);
+		gtk_widget_set_visible(box->seek_bar, TRUE);
+		gtk_widget_set_visible(box->secondary_seek_bar, FALSE);
 	}
 }
 
@@ -480,6 +488,8 @@ set_fullscreen_state(CelluloidControlBox *box, gboolean fullscreen)
 
 	gtk_button_set_icon_name
 		(GTK_BUTTON(box->fullscreen_button), icon_name);
+	gtk_widget_set_visible
+		(box->playlist_button, !fullscreen);
 }
 
 static void
@@ -673,6 +683,12 @@ celluloid_control_box_init(CelluloidControlBox *box)
 {
 	GtkWidget *popup = NULL;
 	GtkAdjustment *adjustment = NULL;
+	const gchar *volume_icons[] =
+		{	"audio-volume-muted",
+			"audio-volume-high",
+			"audio-volume-low",
+			"audio-volume-medium",
+			NULL };
 
 	box->click_gesture = gtk_gesture_click_new();
 
@@ -687,7 +703,7 @@ celluloid_control_box_init(CelluloidControlBox *box)
 	box->fullscreen_button = gtk_button_new();
 	box->loop_button = gtk_toggle_button_new();
 	box->shuffle_button = gtk_toggle_button_new();
-	box->volume_button = gtk_volume_button_new();
+	box->volume_button = gtk_scale_button_new(0.0, 1.0, 0.02, volume_icons);
 	box->seek_bar = celluloid_seek_bar_new();
 	box->secondary_seek_bar = celluloid_seek_bar_new();
 	box->skip_enabled = FALSE;
@@ -731,7 +747,7 @@ celluloid_control_box_init(CelluloidControlBox *box)
 			_("Shuffle Playlist") );
 	init_button(	box->playlist_button,
 			"sidebar-show-right-symbolic",
-			_("Toggle Fullscreen") );
+			_("Toggle Playlist") );
 	init_button(	box->fullscreen_button,
 			"view-fullscreen-symbolic",
 			_("Toggle Fullscreen") );
@@ -739,8 +755,7 @@ celluloid_control_box_init(CelluloidControlBox *box)
 	gtk_widget_add_controller
 		(GTK_WIDGET(box), GTK_EVENT_CONTROLLER(box->click_gesture));
 
-	gtk_style_context_add_class
-		(gtk_widget_get_style_context(GTK_WIDGET(box)), "toolbar");
+	gtk_widget_add_css_class(GTK_WIDGET(box), "toolbar");
 
 	gtk_widget_set_margin_end(box->seek_bar, 6);
 	gtk_widget_set_margin_end(box->secondary_seek_bar, 6);
@@ -758,17 +773,6 @@ celluloid_control_box_init(CelluloidControlBox *box)
 
 	g_object_set(box, "orientation", GTK_ORIENTATION_VERTICAL, NULL);
 
-	gchar css_data[] = "box {border-spacing: 0px;}";
-	GtkCssProvider *css = gtk_css_provider_new();
-	gtk_css_provider_load_from_data(css, css_data, -1);
-
-	GtkStyleContext *style = gtk_widget_get_style_context(GTK_WIDGET(box));
-
-	gtk_style_context_add_provider
-		(	style,
-			GTK_STYLE_PROVIDER(css),
-			GTK_STYLE_PROVIDER_PRIORITY_USER );
-
 	box->inner_box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 
 	gtk_box_append(GTK_BOX(box), box->secondary_seek_bar);
@@ -942,8 +946,7 @@ celluloid_control_box_set_floating(CelluloidControlBox *box, gboolean floating)
 {
 	if(floating)
 	{
-		gtk_style_context_add_class
-			(gtk_widget_get_style_context(GTK_WIDGET(box)), "osd");
+		gtk_widget_add_css_class(GTK_WIDGET(box), "osd");
 
 		gtk_widget_set_margin_start(GTK_WIDGET(box), 12);
 		gtk_widget_set_margin_end(GTK_WIDGET(box), 12);
@@ -952,8 +955,7 @@ celluloid_control_box_set_floating(CelluloidControlBox *box, gboolean floating)
 	}
 	else
 	{
-		gtk_style_context_remove_class
-			(gtk_widget_get_style_context(GTK_WIDGET(box)), "osd");
+		gtk_widget_remove_css_class(GTK_WIDGET(box), "osd");
 
 		gtk_widget_set_margin_start(GTK_WIDGET(box), 0);
 		gtk_widget_set_margin_end(GTK_WIDGET(box), 0);
