@@ -63,9 +63,17 @@ boolean_to_loop(	GBinding *binding,
 			GValue *to_value,
 			gpointer data );
 
+
+static void
+set_video_area_status(	CelluloidController *controller,
+			CelluloidVideoAreaStatus status );
+
 static void
 mpris_enable_handler(GSettings *settings, gchar *key, gpointer data);
 
+static gboolean
+initialize_model(gpointer data);
+
 static void
 view_ready_handler(CelluloidView *view, gpointer data);
 
@@ -73,7 +81,7 @@ static void
 render_handler(CelluloidView *view, gpointer data);
 
 static void
-preferences_updated_handler(CelluloidView *view, gpointer data);
+mpv_reset_request_handler(CelluloidView *view, gpointer data);
 
 static void
 audio_track_load_handler(CelluloidView *view, const gchar *uri, gpointer data);
@@ -257,16 +265,8 @@ constructed(GObject *object)
 	celluloid_controller_input_connect_signals(controller);
 	update_extra_mpv_options(controller);
 
-	// If the window is already realized at this point (happens on X11), the
-	// ready signal will never fire. Because of this, we need to manually
-	// call view_ready_handler() to ensure that the model gets initialized.
-	if(gtk_widget_get_realized(GTK_WIDGET(window)))
-	{
-		view_ready_handler(controller->view, controller);
-	}
-
 	gtk_widget_add_controller(GTK_WIDGET(window), controller->key_controller);
-	gtk_widget_show(GTK_WIDGET(window));
+	gtk_widget_set_visible(GTK_WIDGET(window), TRUE);
 
 	g_signal_connect(	controller->settings,
 				"changed::mpris-enable",
@@ -301,6 +301,12 @@ set_property(	GObject *object,
 
 		case PROP_IDLE:
 		self->idle = g_value_get_boolean(value);
+
+		if(!self->idle)
+		{
+			set_video_area_status
+				(self, CELLULOID_VIDEO_AREA_STATUS_PLAYING);
+		}
 		break;
 
 		case PROP_USE_SKIP_BUTTONS_FOR_PLAYLIST:
@@ -411,6 +417,17 @@ boolean_to_loop(	GBinding *binding,
 	return TRUE;
 }
 
+static void
+set_video_area_status(	CelluloidController *controller,
+			CelluloidVideoAreaStatus status )
+{
+	CelluloidView *view = controller->view;
+	CelluloidMainWindow *window = celluloid_view_get_main_window(view);
+	CelluloidVideoArea *area = celluloid_main_window_get_video_area(window);
+
+	celluloid_video_area_set_status(area, status);
+}
+
 static void
 mpris_enable_handler(GSettings *settings, gchar *key, gpointer data)
 {
@@ -426,23 +443,29 @@ mpris_enable_handler(GSettings *settings, gchar *key, gpointer data)
 	}
 }
 
-static void
-view_ready_handler(CelluloidView *view, gpointer data)
+static gboolean
+initialize_model(gpointer data)
 {
 	CelluloidController *controller = CELLULOID_CONTROLLER(data);
 	CelluloidModel *model = controller->model;
+	CelluloidPlayer *player = CELLULOID_PLAYER(model);
+	CelluloidMpv *mpv = CELLULOID_MPV(model);
+	CelluloidView *view = controller->view;
 	gboolean maximized = FALSE;
 
-	celluloid_player_options_init
-		(	CELLULOID_PLAYER(controller->model),
-			CELLULOID_MAIN_WINDOW(controller->view) );
-	celluloid_model_initialize
-		(model);
+	celluloid_player_options_init(player, CELLULOID_MAIN_WINDOW(view));
+	celluloid_model_initialize(model);
 
-	g_object_get
-		(view, "maximized", &maximized, NULL);
-	celluloid_mpv_set_property_flag
-		(CELLULOID_MPV(model), "window-maximized", maximized);
+	g_object_get(view, "maximized", &maximized, NULL);
+	celluloid_mpv_set_property_flag(mpv, "window-maximized", maximized);
+
+	return G_SOURCE_REMOVE;
+}
+
+static void
+view_ready_handler(CelluloidView *view, gpointer data)
+{
+	g_idle_add(initialize_model, data);
 }
 
 static void
@@ -462,7 +485,7 @@ render_handler(CelluloidView *view, gpointer data)
 }
 
 static void
-preferences_updated_handler(CelluloidView *view, gpointer data)
+mpv_reset_request_handler(CelluloidView *view, gpointer data)
 {
 	CelluloidController *controller = data;
 
@@ -498,18 +521,34 @@ file_open_handler(	CelluloidView *view,
 			gboolean append,
 			gpointer data )
 {
-	CelluloidModel *model = CELLULOID_CONTROLLER(data)->model;
+	CelluloidController *controller = CELLULOID_CONTROLLER(data);
+	CelluloidModel *model = controller->model;
+	const gchar *subtitle_exts[] = SUBTITLE_EXTS;
+	gboolean has_media_file = FALSE;
 	guint files_count = g_list_model_get_n_items(files);
 
+	if(files_count > 0 && !append)
+	{
+		set_video_area_status
+			(controller, CELLULOID_VIDEO_AREA_STATUS_LOADING);
+	}
+
 	for(guint i = 0; i < files_count; i++)
 	{
 		GFile *file = g_list_model_get_item(files, i);
 		gchar *uri = g_file_get_path(file) ?: g_file_get_uri(file);
 
+		has_media_file |= !extension_matches(uri, subtitle_exts);
 		celluloid_model_load_file(model, uri, append || i > 0);
 
 		g_free(uri);
 	}
+
+	if(!has_media_file)
+	{
+		set_video_area_status
+			(controller, CELLULOID_VIDEO_AREA_STATUS_PLAYING);
+	}
 }
 
 static void
@@ -645,9 +684,6 @@ connect_signals(CelluloidController *controller)
 	g_object_bind_property(	controller->model, "core-idle",
 				controller, "idle",
 				G_BINDING_DEFAULT );
-	g_object_bind_property(	controller->model, "border",
-				controller->view, "border",
-				G_BINDING_DEFAULT );
 	g_object_bind_property(	controller->model, "fullscreen",
 				controller->view, "fullscreened",
 				G_BINDING_BIDIRECTIONAL );
@@ -807,8 +843,8 @@ connect_signals(CelluloidController *controller)
 				G_CALLBACK(render_handler),
 				controller );
 	g_signal_connect(	controller->view,
-				"preferences-updated",
-				G_CALLBACK(preferences_updated_handler),
+				"mpv-reset-request",
+				G_CALLBACK(mpv_reset_request_handler),
 				controller );
 	g_signal_connect(	controller->view,
 				"audio-track-load",
@@ -879,19 +915,33 @@ is_more_than_one(	GBinding *binding,
 static void
 idle_active_handler(GObject *object, GParamSpec *pspec, gpointer data)
 {
-	CelluloidController *controller = data;
+	CelluloidController *controller = CELLULOID_CONTROLLER(data);
+	CelluloidModel *model = controller->model;
+	CelluloidView *view = controller->view;
 	gboolean idle_active = TRUE;
 
 	g_object_get(object, "idle-active", &idle_active, NULL);
 
 	if(idle_active)
 	{
-		celluloid_view_reset(CELLULOID_CONTROLLER(data)->view);
+		GPtrArray *playlist = NULL;
+
+		// The "playlist-count" property may not yet be updated at this
+		// point, so we need to use "playlist" directly.
+		g_object_get(model, "playlist", &playlist, NULL);
+
+		celluloid_view_reset(view);
+
+		if(playlist->len <= 0)
+		{
+			set_video_area_status
+				(controller, CELLULOID_VIDEO_AREA_STATUS_IDLE);
+		}
 	}
 	else if(controller->target_playlist_pos >= 0)
 	{
 		celluloid_model_set_playlist_position
-			(controller->model, controller->target_playlist_pos);
+			(model, controller->target_playlist_pos);
 	}
 }
 
@@ -1083,8 +1133,10 @@ message_handler(CelluloidMpv *mpv, const gchar *message, gpointer data)
 static void
 error_handler(CelluloidMpv *mpv, const gchar *message, gpointer data)
 {
-	celluloid_view_show_message_toast
-		(CELLULOID_CONTROLLER(data)->view, message);
+	CelluloidController *controller = CELLULOID_CONTROLLER(data);
+
+	celluloid_view_show_message_toast(controller->view, message);
+	set_video_area_status(controller, CELLULOID_VIDEO_AREA_STATUS_IDLE);
 }
 
 static void
@@ -1421,6 +1473,12 @@ celluloid_controller_open(	CelluloidController *controller,
 				const gchar *uri,
 				gboolean append )
 {
+	if(!append)
+	{
+		set_video_area_status
+			(controller, CELLULOID_VIDEO_AREA_STATUS_LOADING);
+	}
+
 	celluloid_model_load_file(controller->model, uri, append);
 }
 
