@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014-2020 gnome-mpv
+ * Copyright (c) 2014-2021 gnome-mpv
  *
  * This file is part of Celluloid.
  *
@@ -30,25 +30,51 @@
 
 #include <epoxy/gl.h>
 #ifdef GDK_WINDOWING_X11
-#include <gdk/gdkx.h>
+#include <gdk/x11/gdkx.h>
 #include <epoxy/glx.h>
 #endif
 #ifdef GDK_WINDOWING_WAYLAND
-#include <gdk/gdkwayland.h>
+#include <gdk/wayland/gdkwayland.h>
 #include <epoxy/egl.h>
 #endif
 #ifdef GDK_WINDOWING_WIN32
-#include <gdk/gdkwin32.h>
+#include <gdk/win32/gdkwin32.h>
 #include <epoxy/wgl.h>
 #endif
 
 #include "celluloid-mpv.h"
-#include "celluloid-mpv-private.h"
-#include "celluloid-mpv-wrapper.h"
 #include "celluloid-common.h"
 #include "celluloid-def.h"
 #include "celluloid-marshal.h"
 
+#define get_private(mpv) \
+	((CelluloidMpvPrivate *)celluloid_mpv_get_instance_private(mpv))
+
+typedef struct _CelluloidMpvPrivate CelluloidMpvPrivate;
+
+enum
+{
+	PROP_0,
+	PROP_WID,
+	PROP_READY,
+	N_PROPERTIES
+};
+
+struct _CelluloidMpvPrivate
+{
+	mpv_handle *mpv_ctx;
+	mpv_render_context *render_ctx;
+	gboolean ready;
+	gchar *tmp_input_file;
+	GSList *log_level_list;
+	gboolean init_vo_config;
+	gboolean force_opengl;
+	gboolean use_opengl;
+	gint64 wid;
+	void *render_update_callback_data;
+	void (*render_update_callback)(void *data);
+};
+
 static void *
 get_proc_address(void *fn_ctx, const gchar *name);
 
@@ -319,13 +345,20 @@ check_mpv_version(const gchar *version)
 	gchar **tokens = NULL;
 	gboolean done = FALSE;
 	gboolean result = TRUE;
+	guint version_offset = 0;
 
-	/* Skip to the version number */
-	if(strncmp(version, "mpv ", 4) == 0)
+	if(strncmp(version, "mpv v", 5) == 0)
 	{
-		tokens = g_strsplit(version+4, ".", (gint)min_version_length);
+		version_offset = 5;
+	}
+	else if(strncmp(version, "mpv ", 4) == 0)
+	{
+		version_offset = 4;
 	}
 
+	/* Skip to the version number */
+	const gchar *trimmed_version = version + version_offset;
+	tokens = g_strsplit(trimmed_version, ".", (gint)min_version_length);
 	done = !tokens || g_strv_length(tokens) != min_version_length;
 	result = !done;
 
@@ -387,20 +420,15 @@ initialize(CelluloidMpv *mpv)
 	gchar *current_vo = NULL;
 	gchar *mpv_version = NULL;
 
-	if(priv->wid < 0)
-	{
-		g_info("Forcing --vo=libmpv");
-		mpv_set_option_string(priv->mpv_ctx, "vo", "libmpv");
-	}
-	else if(priv->wid == 0)
+	if(priv->wid == 0)
 	{
 		g_info("Forcing --vo=null");
 		mpv_set_option_string(priv->mpv_ctx, "vo", "null");
 	}
 	else
 	{
-		g_debug("Attaching mpv window to wid %#x", (guint)priv->wid);
-		mpv_set_option(priv->mpv_ctx, "wid", MPV_FORMAT_INT64, &priv->wid);
+		g_info("Forcing --vo=libmpv");
+		mpv_set_option_string(priv->mpv_ctx, "vo", "libmpv");
 	}
 
 	mpv_set_wakeup_callback(priv->mpv_ctx, wakeup_callback, mpv);
@@ -778,3 +806,273 @@ celluloid_mpv_load(CelluloidMpv *mpv, const gchar *uri, gboolean append)
 		celluloid_mpv_load_file(mpv, uri, append);
 	}
 }
+
+gint
+celluloid_mpv_command(CelluloidMpv *mpv, const gchar **cmd)
+{
+	CelluloidMpvPrivate *priv = get_private(mpv);
+	gint rc = MPV_ERROR_UNINITIALIZED;
+
+	if(priv->mpv_ctx)
+	{
+		rc = mpv_command(priv->mpv_ctx, cmd);
+	}
+
+	if(rc < 0)
+	{
+		gchar *cmd_str = g_strjoinv(" ", (gchar **)cmd);
+
+		g_warning(	"Failed to run mpv command \"%s\". Reason: %s.",
+				cmd_str,
+				mpv_error_string(rc) );
+
+		g_free(cmd_str);
+	}
+
+	return rc;
+}
+
+gint
+celluloid_mpv_command_async(CelluloidMpv *mpv, const gchar **cmd)
+{
+	CelluloidMpvPrivate *priv = get_private(mpv);
+	gint rc = MPV_ERROR_UNINITIALIZED;
+
+	if(priv->mpv_ctx)
+	{
+		rc = mpv_command_async(priv->mpv_ctx, 0, cmd);
+	}
+
+	if(rc < 0)
+	{
+		gchar *cmd_str = g_strjoinv(" ", (gchar **)cmd);
+
+		g_warning(	"Failed to dispatch async mpv command \"%s\". "
+				"Reason: %s.",
+				cmd_str,
+				mpv_error_string(rc) );
+
+		g_free(cmd_str);
+	}
+
+	return rc;
+}
+
+gint
+celluloid_mpv_command_string(CelluloidMpv *mpv, const gchar *cmd)
+{
+	CelluloidMpvPrivate *priv = get_private(mpv);
+	gint rc = MPV_ERROR_UNINITIALIZED;
+
+	if(priv->mpv_ctx)
+	{
+		rc = mpv_command_string(priv->mpv_ctx, cmd);
+	}
+
+	if(rc < 0)
+	{
+		g_warning(	"Failed to run mpv command string \"%s\". "
+				"Reason: %s.",
+				cmd,
+				mpv_error_string(rc) );
+	}
+
+	return rc;
+}
+
+gint
+celluloid_mpv_set_option_string(	CelluloidMpv *mpv,
+					const gchar *name,
+					const gchar *value )
+{
+	return mpv_set_option_string(get_private(mpv)->mpv_ctx, name, value);
+}
+
+gint
+celluloid_mpv_get_property(	CelluloidMpv *mpv,
+				const gchar *name,
+				mpv_format format,
+				void *data )
+{
+	CelluloidMpvPrivate *priv = get_private(mpv);
+	gint rc = MPV_ERROR_UNINITIALIZED;
+
+	if(priv->mpv_ctx)
+	{
+		rc = mpv_get_property(priv->mpv_ctx, name, format, data);
+	}
+
+	if(rc < 0)
+	{
+		g_info(	"Failed to retrieve property \"%s\" "
+			"using mpv format %d. Reason: %s.",
+			name,
+			format,
+			mpv_error_string(rc) );
+	}
+
+	return rc;
+}
+
+gchar *
+celluloid_mpv_get_property_string(CelluloidMpv *mpv, const gchar *name)
+{
+	CelluloidMpvPrivate *priv = get_private(mpv);
+	gchar *value = NULL;
+
+	if(priv->mpv_ctx)
+	{
+		value = mpv_get_property_string(priv->mpv_ctx, name);
+	}
+
+	if(!value)
+	{
+		g_info("Failed to retrieve property \"%s\" as string.", name);
+	}
+
+	return value;
+}
+
+gboolean
+celluloid_mpv_get_property_flag(CelluloidMpv *mpv, const gchar *name)
+{
+	CelluloidMpvPrivate *priv = get_private(mpv);
+	gboolean value = FALSE;
+	gint rc = MPV_ERROR_UNINITIALIZED;
+
+	if(priv->mpv_ctx)
+	{
+		rc =	mpv_get_property
+			(priv->mpv_ctx, name, MPV_FORMAT_FLAG, &value);
+	}
+
+	if(rc < 0)
+	{
+		g_info(	"Failed to retrieve property \"%s\" as flag. "
+			"Reason: %s.",
+			name,
+			mpv_error_string(rc) );
+	}
+
+	return value;
+}
+
+gint
+celluloid_mpv_set_property(	CelluloidMpv *mpv,
+				const gchar *name,
+				mpv_format format,
+				void *data )
+{
+	CelluloidMpvPrivate *priv = get_private(mpv);
+	gint rc = MPV_ERROR_UNINITIALIZED;
+
+	if(priv->mpv_ctx)
+	{
+		rc = mpv_set_property(priv->mpv_ctx, name, format, data);
+	}
+
+	if(rc < 0)
+	{
+		g_info(	"Failed to set property \"%s\" using mpv format %d. "
+			"Reason: %s.",
+			name,
+			format,
+			mpv_error_string(rc) );
+	}
+
+	return rc;
+}
+
+gint
+celluloid_mpv_set_property_string(	CelluloidMpv *mpv,
+					const gchar *name,
+					const char *data )
+{
+	CelluloidMpvPrivate *priv = get_private(mpv);
+	gint rc = MPV_ERROR_UNINITIALIZED;
+
+	if(priv->mpv_ctx)
+	{
+		rc = mpv_set_property_string(priv->mpv_ctx, name, data);
+	}
+
+	if(rc < 0)
+	{
+		g_info(	"Failed to set property \"%s\" as string. Reason: %s.",
+			name,
+			mpv_error_string(rc) );
+	}
+
+	return rc;
+}
+
+gint
+celluloid_mpv_set_property_flag(	CelluloidMpv *mpv,
+					const gchar *name,
+					gboolean value )
+{
+	CelluloidMpvPrivate *priv = get_private(mpv);
+	gint rc = MPV_ERROR_UNINITIALIZED;
+
+	if(priv->mpv_ctx)
+	{
+		rc =	mpv_set_property
+			(priv->mpv_ctx, name, MPV_FORMAT_FLAG, &value);
+	}
+
+	if(rc < 0)
+	{
+		g_info(	"Failed to set property \"%s\" as flag. Reason: %s.",
+			name,
+			mpv_error_string(rc) );
+	}
+
+	return rc;
+}
+
+void
+celluloid_mpv_set_render_update_callback(	CelluloidMpv *mpv,
+						mpv_render_update_fn func,
+						void *data )
+{
+	CelluloidMpvPrivate *priv = get_private(mpv);
+
+	priv->render_update_callback = func;
+	priv->render_update_callback_data = data;
+
+	if(priv->render_ctx)
+	{
+		mpv_render_context_set_update_callback
+			(priv->render_ctx, func, data);
+	}
+}
+
+guint64
+celluloid_mpv_render_context_update(CelluloidMpv *mpv)
+{
+	return mpv_render_context_update(get_private(mpv)->render_ctx);
+}
+
+gint
+celluloid_mpv_load_config_file(CelluloidMpv *mpv, const gchar *filename)
+{
+	return mpv_load_config_file(get_private(mpv)->mpv_ctx, filename);
+}
+
+gint
+celluloid_mpv_observe_property(	CelluloidMpv *mpv,
+				guint64 reply_userdata,
+				const gchar *name,
+				mpv_format format )
+{
+	return mpv_observe_property(	get_private(mpv)->mpv_ctx,
+					reply_userdata,
+					name,
+					format );
+}
+
+gint
+celluloid_mpv_request_log_messages(CelluloidMpv *mpv, const gchar *min_level)
+{
+	return mpv_request_log_messages(get_private(mpv)->mpv_ctx, min_level);
+}
