@@ -54,7 +54,6 @@ struct _CelluloidPlaylistWidget
 	GtkBox parent_instance;
 	gint64 playlist_count;
 	gboolean searching;
-	GtkListStore *store;
 	GtkWidget *scrolled_window;
 	CelluloidPlaylistModel *model;
 	GtkWidget *list_box;
@@ -292,10 +291,6 @@ constructed(GObject *object)
 {
 	CelluloidPlaylistWidget *self = CELLULOID_PLAYLIST_WIDGET(object);
 
-	self->store = gtk_list_store_new(	3,
-						G_TYPE_STRING,
-						G_TYPE_STRING,
-						G_TYPE_INT );
 	self->model = celluloid_playlist_model_new();
 	self->list_box = gtk_list_box_new();
 	self->last_selected = -1;
@@ -600,7 +595,7 @@ pressed_handler(	GtkGestureClick *gesture,
 		gtk_popover_set_has_arrow(GTK_POPOVER(ctx_menu), FALSE);
 		gtk_popover_set_pointing_to(GTK_POPOVER(ctx_menu), &rect);
 		gtk_widget_set_halign(ctx_menu, GTK_ALIGN_START);
-		gtk_widget_show(ctx_menu);
+		gtk_widget_set_visible(ctx_menu, TRUE);
 	}
 
 	return FALSE;
@@ -908,8 +903,6 @@ celluloid_playlist_widget_class_init(CelluloidPlaylistWidgetClass *klass)
 static void
 celluloid_playlist_widget_init(CelluloidPlaylistWidget *wgt)
 {
-	GtkStyleContext *style = NULL;
-
 	wgt->playlist_count = 0;
 	wgt->searching = FALSE;
 	wgt->scrolled_window = gtk_scrolled_window_new();
@@ -918,8 +911,7 @@ celluloid_playlist_widget_init(CelluloidPlaylistWidget *wgt)
 	wgt->placeholder = gtk_label_new(_("Playlist is empty"));
 	wgt->css_provider = gtk_css_provider_new();
 
-	style = gtk_widget_get_style_context(wgt->placeholder);
-	gtk_style_context_add_class(style, "dim-label");
+	gtk_widget_add_css_class(wgt->placeholder, "dim-label");
 
 	gtk_widget_set_vexpand
 		(wgt->scrolled_window, TRUE);
