@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016-2021 gnome-mpv
+ * Copyright (c) 2016-2021, 2023 gnome-mpv
  *
  * This file is part of Celluloid.
  *
@@ -187,9 +187,13 @@ add_handler(GtkButton *button, gpointer data)
 	GtkFileFilter *filter;
 	GtkFileChooser *chooser;
 
-	dialog = celluloid_file_chooser_new(	_("Add Plugin"),
-						pmgr->parent_window,
-						GTK_FILE_CHOOSER_ACTION_OPEN );
+	dialog =
+		celluloid_file_chooser_new
+		(	_("Add Plugin"),
+			pmgr->parent_window,
+			GTK_FILE_CHOOSER_ACTION_OPEN,
+			TRUE );
+
 	filter = NULL;
 	chooser = GTK_FILE_CHOOSER(dialog);
 
@@ -283,31 +287,28 @@ changed_handler(	GFileMonitor *monitor,
 		}
 
 		// Populate the list box with files in the plugins directory
-		do
+		filename = g_dir_read_name(dir);
+
+		while(filename)
 		{
-			gchar *full_path;
+			gchar *full_path =
+				g_build_filename
+				(pmgr->directory, filename, NULL);
+			GtkWidget *item =
+				celluloid_plugins_manager_item_new
+				(	pmgr->parent_window,
+					filename,
+					full_path );
 
-			filename = g_dir_read_name(dir);
-			full_path =	g_build_filename
-					(pmgr->directory, filename, NULL);
+			adw_preferences_group_add
+				(pmgr->pref_group, item);
 
-			if(g_file_test(full_path, G_FILE_TEST_IS_REGULAR))
-			{
-				GtkWidget *item;
-
-				item =	celluloid_plugins_manager_item_new
-					(	pmgr->parent_window,
-						filename,
-						full_path );
-				adw_preferences_group_add
-					(pmgr->pref_group, item);
-
-				empty = FALSE;
-			}
+			empty = FALSE;
 
 			g_free(full_path);
+
+			filename = g_dir_read_name(dir);
 		}
-		while(filename);
 
 		gtk_widget_set_visible(pmgr->placeholder, empty);
 
@@ -368,7 +369,7 @@ copy_file_to_directory(CelluloidPluginsManager *pmgr, GFile *src)
 				dest_path,
 				error->message );
 
-		gtk_widget_show(error_dialog);
+		gtk_widget_set_visible(error_dialog, TRUE);
 
 		g_error_free(error);
 		g_free(src_path);
