@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014-2022 gnome-mpv
+ * Copyright (c) 2014-2024 gnome-mpv
  *
  * This file is part of Celluloid.
  *
@@ -37,6 +37,7 @@ struct _CelluloidPreferencesDialog
 {
 	AdwPreferencesWindow parent_instance;
 	GSettings *settings;
+	gboolean needs_mpv_reset;
 };
 
 struct _CelluloidPreferencesDialogClass
@@ -76,12 +77,31 @@ file_set_handler(CelluloidFileChooserButton *button, gpointer data)
 	g_object_unref(file);
 }
 
+static void
+handle_changed(GSettings *settings, const gchar *key, gpointer data)
+{
+	CelluloidPreferencesDialog *dlg = CELLULOID_PREFERENCES_DIALOG(data);
+
+	dlg->needs_mpv_reset |= g_strcmp0(key, "mpv-config-enable") == 0;
+	dlg->needs_mpv_reset |= g_strcmp0(key, "mpv-config-file") == 0;
+	dlg->needs_mpv_reset |= g_strcmp0(key, "mpv-input-config-enable") == 0;
+	dlg->needs_mpv_reset |= g_strcmp0(key, "mpv-input-config-file") == 0;
+	dlg->needs_mpv_reset |= g_strcmp0(key, "mpv-options") == 0;
+}
+
 static gboolean
 save_settings(AdwPreferencesWindow *dialog)
 {
 	CelluloidPreferencesDialog *dlg = CELLULOID_PREFERENCES_DIALOG(dialog);
+
 	g_settings_apply(dlg->settings);
 
+	if(dlg->needs_mpv_reset)
+	{
+		g_signal_emit_by_name(dlg, "mpv-reset-request");
+		dlg->needs_mpv_reset = FALSE;
+	}
+
 	return FALSE;
 }
 
@@ -245,6 +265,16 @@ celluloid_preferences_dialog_class_init(CelluloidPreferencesDialogClass *klass)
 {
 	G_OBJECT_CLASS(klass)->constructed = preferences_dialog_constructed;
 	G_OBJECT_CLASS(klass)->finalize = finalize;
+
+	g_signal_new(	"mpv-reset-request",
+			G_TYPE_FROM_CLASS(klass),
+			G_SIGNAL_RUN_FIRST,
+			0,
+			NULL,
+			NULL,
+			g_cclosure_marshal_VOID__VOID,
+			G_TYPE_NONE,
+			0 );
 }
 
 static void
@@ -264,6 +294,9 @@ celluloid_preferences_dialog_init(CelluloidPreferencesDialog *dlg)
 			"always-use-floating-controls",
 			ITEM_TYPE_SWITCH},
 			{NULL,
+			"always-use-floating-header-bar",
+			ITEM_TYPE_SWITCH},
+			{NULL,
 			"draggable-video-area-enable",
 			ITEM_TYPE_SWITCH},
 			{NULL,
@@ -286,13 +319,13 @@ celluloid_preferences_dialog_init(CelluloidPreferencesDialog *dlg)
 		= {	{NULL,
 			"mpv-config-enable",
 			ITEM_TYPE_SWITCH},
-			{_("mpv configuration file:"),
+			{_("mpv configuration file"),
 			"mpv-config-file",
 			ITEM_TYPE_FILE_CHOOSER},
 			{NULL,
 			"mpv-input-config-enable",
 			ITEM_TYPE_SWITCH},
-			{_("mpv input configuration file:"),
+			{_("mpv input configuration file"),
 			"mpv-input-config-file",
 			ITEM_TYPE_FILE_CHOOSER},
 			{NULL, NULL, ITEM_TYPE_INVALID} };
@@ -318,6 +351,8 @@ celluloid_preferences_dialog_init(CelluloidPreferencesDialog *dlg)
 			{NULL, NULL, ITEM_TYPE_INVALID} };
 
 	dlg->settings = g_settings_new(CONFIG_ROOT);
+	dlg->needs_mpv_reset = FALSE;
+
 	g_settings_delay(dlg->settings);
 
 	GtkWidget *page = NULL;
@@ -326,7 +361,7 @@ celluloid_preferences_dialog_init(CelluloidPreferencesDialog *dlg)
 		(	interface_items,
 			dlg->settings,
 			_("Interface"),
-			"preferences-desktop-appearance-symbolic" );
+			"applications-graphics-symbolic" );
 	adw_preferences_window_add(	ADW_PREFERENCES_WINDOW(dlg),
 					ADW_PREFERENCES_PAGE(page));
 
@@ -354,6 +389,10 @@ celluloid_preferences_dialog_init(CelluloidPreferencesDialog *dlg)
 				"close-request",
 				G_CALLBACK(save_settings),
 				NULL );
+	g_signal_connect(	dlg->settings,
+				"changed",
+				G_CALLBACK(handle_changed),
+				dlg );
 }
 
 GtkWidget *
@@ -368,7 +407,7 @@ celluloid_preferences_dialog_new(GtkWindow *parent)
 				"transient-for", parent,
 				NULL );
 
-	gtk_widget_show(dlg);
+	gtk_widget_set_visible(dlg, TRUE);
 
 	return dlg;
 }
