@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016-2022 gnome-mpv
+ * Copyright (c) 2016-2022, 2024 gnome-mpv
  *
  * This file is part of Celluloid.
  *
@@ -31,6 +31,15 @@
 
 G_DECLARE_FINAL_TYPE(CelluloidVideoArea, celluloid_video_area, CELLULOID, VIDEO_AREA, GtkBox)
 
+enum CelluloidVideoAreaStatus
+{
+	CELLULOID_VIDEO_AREA_STATUS_LOADING,
+	CELLULOID_VIDEO_AREA_STATUS_IDLE,
+	CELLULOID_VIDEO_AREA_STATUS_PLAYING
+};
+
+typedef enum CelluloidVideoAreaStatus CelluloidVideoAreaStatus;
+
 GtkWidget *
 celluloid_video_area_new(void);
 
@@ -55,12 +64,16 @@ celluloid_video_area_set_control_box_visible(	CelluloidVideoArea *area,
 						gboolean visible );
 
 void
-celluloid_video_area_set_initial_page_visible(	CelluloidVideoArea *area,
-						gboolean visible );
+celluloid_video_area_set_status(	CelluloidVideoArea *area,
+					CelluloidVideoAreaStatus status );
 
 gboolean
 celluloid_video_area_get_control_box_visible(CelluloidVideoArea *area);
 
+void
+celluloid_video_area_set_use_floating_header_bar(	CelluloidVideoArea *area,
+							gboolean floating );
+
 void
 celluloid_video_area_queue_render(CelluloidVideoArea *area);
 
