@@ -52,7 +52,6 @@ enum
 	PROP_SKIP_ENABLED,
 	PROP_LOOP,
 	PROP_SHUFFLE,
-	PROP_BORDER,
 	PROP_MEDIA_TITLE,
 	PROP_DISPLAY_FPS,
 	PROP_SEARCHING,
@@ -80,7 +79,6 @@ struct _CelluloidView
 	gboolean control_box_enabled;
 	gboolean loop;
 	gboolean shuffle;
-	gboolean border;
 	gchar *media_title;
 	gdouble display_fps;
 	gboolean searching;
@@ -152,18 +150,20 @@ open_track_dialog_response_handler(	GtkDialog *dialog,
 					gint response_id,
 					gpointer data );
 
-static gboolean
-preferences_dialog_response_handler(	AdwPreferencesWindow *dialog,
-					gpointer data );
-
 static void
 save_playlist_response_handler(	GtkDialog *dialog,
 				gint response_id,
 				gpointer data );
 
+static gboolean
+mpv_reset_request_handler(AdwPreferencesWindow *dialog, gpointer data);
+
 static void
 realize_handler(GtkWidget *widget, gpointer data);
 
+static void
+notify_mapped_handler(GdkSurface *surface, GParamSpec *pspec,  gpointer data);
+
 static void
 resize_handler(	CelluloidVideoArea *video_area,
 		gint width,
@@ -338,8 +338,6 @@ set_property(	GObject *object,
 		CELLULOID_MAIN_WINDOW(self);
 	CelluloidControlBox* control_box =
 		celluloid_main_window_get_control_box(wnd);
-	CelluloidVideoArea *area =
-		celluloid_main_window_get_video_area(wnd);
 
 	switch(property_id)
 	{
@@ -366,11 +364,6 @@ set_property(	GObject *object,
 		case PROP_IDLE_ACTIVE:
 		self->idle_active = g_value_get_boolean(value);
 		update_title(self);
-		if(!self->idle_active)
-		{
-			celluloid_video_area_set_initial_page_visible
-				(area, FALSE);
-		}
 		break;
 
 		case PROP_VOLUME:
@@ -422,21 +415,6 @@ set_property(	GObject *object,
 		self->shuffle = g_value_get_boolean(value);
 		break;
 
-		case PROP_BORDER:
-		self->border = g_value_get_boolean(value);
-		{
-			GtkWidget *titlebar =
-				gtk_window_get_titlebar(GTK_WINDOW(wnd));
-
-			if(titlebar)
-			{
-				gtk_widget_set_visible(titlebar, self->border);
-			}
-
-			gtk_window_set_decorated(GTK_WINDOW(wnd), self->border);
-		}
-		break;
-
 		case PROP_MEDIA_TITLE:
 		g_free(self->media_title);
 		self->media_title = g_value_dup_string(value);
@@ -530,10 +508,6 @@ get_property(	GObject *object,
 		g_value_set_boolean(value, self->shuffle);
 		break;
 
-		case PROP_BORDER:
-		g_value_set_boolean(value, self->border);
-		break;
-
 		case PROP_MEDIA_TITLE:
 		g_value_set_static_string(value, self->media_title);
 		break;
@@ -750,7 +724,7 @@ show_message_dialog(	CelluloidView *view,
 
 		gtk_window_set_title(GTK_WINDOW(dialog), title);
 		gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
-		gtk_widget_show(dialog);
+		gtk_widget_set_visible(dialog, TRUE);
 	}
 }
 
@@ -781,7 +755,8 @@ show_open_track_dialog(CelluloidView  *view, TrackType type)
 	CelluloidFileChooser *chooser =	celluloid_file_chooser_new
 					(	title,
 						GTK_WINDOW(view),
-						GTK_FILE_CHOOSER_ACTION_OPEN );
+						GTK_FILE_CHOOSER_ACTION_OPEN,
+						TRUE );
 
 	g_object_set_data(	G_OBJECT(chooser),
 				"track-type",
@@ -921,38 +896,6 @@ open_track_dialog_response_handler(	GtkDialog *dialog,
 	celluloid_file_chooser_destroy(CELLULOID_FILE_CHOOSER(dialog));
 }
 
-static gboolean
-preferences_dialog_response_handler(	AdwPreferencesWindow *dialog,
-					gpointer data )
-{
-	CelluloidMainWindow *wnd;
-	GSettings *settings;
-	gboolean csd_enable;
-
-	wnd = CELLULOID_MAIN_WINDOW(data);
-	settings = g_settings_new(CONFIG_ROOT);
-	csd_enable = g_settings_get_boolean(settings, "csd-enable");
-
-	if(celluloid_main_window_get_csd_enabled(wnd) != csd_enable)
-	{
-		show_message_dialog(	CELLULOID_VIEW(data),
-					GTK_MESSAGE_INFO,
-					g_get_application_name(),
-					NULL,
-					_("Enabling or disabling "
-					"client-side decorations "
-					"requires restarting to "
-					"take effect.") );
-	}
-
-	gtk_widget_queue_draw(GTK_WIDGET(wnd));
-	g_signal_emit_by_name(data, "preferences-updated");
-
-	g_object_unref(settings);
-
-	return FALSE;
-}
-
 static void
 save_playlist_response_handler(	GtkDialog *dialog,
 				gint response_id,
@@ -988,12 +931,59 @@ save_playlist_response_handler(	GtkDialog *dialog,
 	}
 }
 
+static gboolean
+mpv_reset_request_handler(AdwPreferencesWindow *dialog, gpointer data)
+{
+	CelluloidMainWindow *wnd;
+	GSettings *settings;
+	gboolean csd_enable;
+
+	wnd = CELLULOID_MAIN_WINDOW(data);
+	settings = g_settings_new(CONFIG_ROOT);
+	csd_enable = g_settings_get_boolean(settings, "csd-enable");
+
+	if(celluloid_main_window_get_csd_enabled(wnd) != csd_enable)
+	{
+		show_message_dialog(	CELLULOID_VIEW(data),
+					GTK_MESSAGE_INFO,
+					g_get_application_name(),
+					NULL,
+					_("Enabling or disabling "
+					"client-side decorations "
+					"requires restarting to "
+					"take effect.") );
+	}
+
+	gtk_widget_queue_draw(GTK_WIDGET(wnd));
+	g_signal_emit_by_name(data, "mpv-reset-request");
+
+	g_object_unref(settings);
+
+	return FALSE;
+}
+
 static void
 realize_handler(GtkWidget *widget, gpointer data)
 {
-	load_css(CELLULOID_VIEW(widget));
+	CelluloidView *view = CELLULOID_VIEW(widget);
+	GdkSurface *surface = gtk_widget_get_surface(view);
 
-	g_signal_emit_by_name(CELLULOID_VIEW(widget), "ready");
+	g_signal_connect(	surface,
+				"notify::mapped",
+				G_CALLBACK(notify_mapped_handler),
+				widget );
+
+	load_css(view);
+}
+
+static void
+notify_mapped_handler(GdkSurface *surface, GParamSpec *pspec, gpointer data)
+{
+	g_signal_handlers_disconnect_by_func(	surface,
+						notify_mapped_handler,
+						data );
+
+	g_signal_emit_by_name(CELLULOID_VIEW(data), "ready");
 }
 
 static void
@@ -1024,6 +1014,8 @@ drop_handler(	GtkDropTarget *self,
 	const gboolean append = CELLULOID_IS_PLAYLIST_WIDGET(source);
 	GListStore *files = NULL;
 
+	g_debug("Received drop event with value type %s", g_type_name(G_VALUE_TYPE(value)) );
+
 	if(G_VALUE_HOLDS_STRING(value))
 	{
 		const gchar *string = g_value_get_string(value);
@@ -1043,6 +1035,10 @@ drop_handler(	GtkDropTarget *self,
 
 			g_strfreev(uris);
 		}
+		else
+		{
+			g_warning("Failed to extract any URIs from dropped string");
+		}
 	}
 	else if(G_VALUE_HOLDS(value, GDK_TYPE_FILE_LIST))
 	{
@@ -1055,12 +1051,21 @@ drop_handler(	GtkDropTarget *self,
 			g_list_store_append(files, G_FILE(cur->data));
 		}
 	}
+	else
+	{
+		g_error(	"Cannot handle drop event with value type %s",
+				g_type_name(G_VALUE_TYPE(value)) );
+	}
 
 	if(files)
 	{
 		g_signal_emit_by_name(view, "file-open", files, append);
 		g_object_unref(files);
 	}
+	else
+	{
+		g_warning("Failed to open anything from drop event");
+	}
 
 	return TRUE;
 }
@@ -1236,14 +1241,6 @@ celluloid_view_class_init(CelluloidViewClass *klass)
 			G_PARAM_READWRITE );
 	g_object_class_install_property(object_class, PROP_SHUFFLE, pspec);
 
-	pspec = g_param_spec_boolean
-		(	"border",
-			"Border",
-			"Whether or not the main window should have decorations",
-			FALSE,
-			G_PARAM_READWRITE );
-	g_object_class_install_property(object_class, PROP_BORDER, pspec);
-
 	pspec = g_param_spec_string
 		(	"media-title",
 			"Media Title",
@@ -1301,7 +1298,7 @@ celluloid_view_class_init(CelluloidViewClass *klass)
 			g_cclosure_marshal_VOID__VOID,
 			G_TYPE_NONE,
 			0 );
-	g_signal_new(	"preferences-updated",
+	g_signal_new(	"mpv-reset-request",
 			G_TYPE_FROM_CLASS(klass),
 			G_SIGNAL_RUN_FIRST,
 			0,
@@ -1412,7 +1409,6 @@ celluloid_view_init(CelluloidView *view)
 	view->skip_enabled = FALSE;
 	view->loop = FALSE;
 	view->shuffle = FALSE;
-	view->border = FALSE;
 	view->media_title = NULL;
 	view->display_fps = 0;
 	view->searching = FALSE;
@@ -1478,7 +1474,10 @@ celluloid_view_show_open_dialog(	CelluloidView *view,
 		action = GTK_FILE_CHOOSER_ACTION_OPEN;
 	}
 
-	chooser = celluloid_file_chooser_new(title, GTK_WINDOW(view), action);
+	chooser =
+		celluloid_file_chooser_new
+		(title, GTK_WINDOW(view), action, TRUE);
+
 	args = g_ptr_array_new();
 	append_buf = g_malloc(sizeof(gboolean));
 	*append_buf = append;
@@ -1525,7 +1524,7 @@ celluloid_view_show_open_location_dialog(CelluloidView *view, gboolean append)
 			G_CALLBACK(open_location_dialog_response_handler),
 			args );
 
-	gtk_widget_show(dlg);
+	gtk_widget_set_visible(dlg, TRUE);
 }
 
 void
@@ -1555,7 +1554,8 @@ celluloid_view_show_save_playlist_dialog(CelluloidView *view)
 	file_chooser =	celluloid_file_chooser_new
 			(	_("Save Playlist"),
 				GTK_WINDOW(view),
-				GTK_FILE_CHOOSER_ACTION_SAVE );
+				GTK_FILE_CHOOSER_ACTION_SAVE,
+				TRUE );
 	gtk_chooser = GTK_FILE_CHOOSER(file_chooser);
 
 	gtk_file_chooser_set_current_name(gtk_chooser, "playlist.m3u");
@@ -1574,11 +1574,11 @@ celluloid_view_show_preferences_dialog(CelluloidView *view)
 	GtkWidget *dialog = celluloid_preferences_dialog_new(GTK_WINDOW(view));
 
 	g_signal_connect_after(	dialog,
-				"close-request",
-				G_CALLBACK(preferences_dialog_response_handler),
+				"mpv-reset-request",
+				G_CALLBACK(mpv_reset_request_handler),
 				view );
 
-	gtk_widget_show(dialog);
+	gtk_widget_set_visible(dialog, TRUE);
 }
 
 void
@@ -1586,7 +1586,7 @@ celluloid_view_show_shortcuts_dialog(CelluloidView *view)
 {
 	GtkWidget *wnd = celluloid_shortcuts_window_new(GTK_WINDOW(view));
 
-	gtk_widget_show(wnd);
+	gtk_widget_set_visible(wnd, TRUE);
 }
 
 void
